/**************************************************************************

Copyright (c) 2002 Dmitry Vukolov <dvukol@rosmail.ru>

This is free software, licensed under the GNU Public License v2.
See the file COPYING for details.

**************************************************************************/

#include <PalmOS.h>
#include "convert.h"
#include "commondb.h"
#include "resource.h"

#define MEM_BUF_SIZE    1000

#define ENC_RES_TYPE    'enco'
#define LOWER_BOUND     128
#define UPPER_BOUND     256
#define CONV_TABLE_SIZE (UPPER_BOUND-LOWER_BOUND)
#define NO_RECODE       0

UInt8 *ConvTable;
MemHandle MemBuf;
Char **EncNames = NULL;

UInt16 ConvertStr(Char * str, UInt16 strLen, Char * strDest);

UInt16 EncTotal()
{
	if (EncNames)
		return MemPtrSize(EncNames) / sizeof(EncNames[0]);
	return 0;
}

Char **GetEncNames()
{
	UInt16 index, resIndex = 0, numEncs;
	MemHandle recordH;
	MemPtr recordP;
	DmOpenRef dbP;

	if (!EncNames) {
		dbP = DmNextOpenResDatabase(NULL);
		for (index = 0; resIndex != (UInt16)-1; index++) {
			resIndex =
			    DmFindResourceType(dbP, ENC_RES_TYPE, index);
		}
		numEncs = index - 1;
		if (numEncs == 0) {
			FrmAlert(NoEncsAlert);
			return NULL;
		}
		EncNames = (Char **) MemPtrNew(numEncs * sizeof(Char *));
		if (EncNames == NULL) {
			FrmAlert(NoMemAlert);
			return NULL;
		}
		for (index = 0; index < numEncs; index++) {
			resIndex =
			    DmFindResourceType(dbP, ENC_RES_TYPE, index);
			recordH = DmGetResourceIndex(dbP, resIndex);
			if (recordH == NULL) {
				FrmAlert(ErrOpenEncAlert);
				return NULL;
			}
			recordP = MemHandleLock(recordH);
			EncNames[index] =
			    (Char *) MemPtrNew(StrLen((Char *) recordP) +
					       1);
			if (EncNames[index] == NULL) {
				FrmAlert(NoMemAlert);
				return NULL;
			}
			StrCopy(EncNames[index], (Char *) recordP);
			MemPtrUnlock(recordP);
		}
	}
	return EncNames;
}

void FreeEncNames()
{
	UInt16 numEncs, index;

	if (EncNames) {
		numEncs = EncTotal();
		for (index = 0; index < numEncs; index++) {
			if (EncNames[index])
				MemPtrFree(EncNames[index]);
		}
		MemPtrFree(EncNames);
	}
}

Boolean ConverterInit(UInt8 fromCharsetIndex, UInt8 toCharsetIndex)
{
	MemHandle fromCharsetH, toCharsetH;
	UInt32 CharsetSize, CharsetSize2, fromNameSize, toNameSize;
	UInt8 *fromCharsetP, *toCharsetP, *p, *pf, *pt;
	UInt8 index;
	DmOpenRef dbP;

	ConverterFree();

	dbP = DmNextOpenResDatabase(NULL);
	fromCharsetH =
	    DmGetResourceIndex(dbP,
			       DmFindResourceType(dbP, ENC_RES_TYPE,
						  fromCharsetIndex));
	if (fromCharsetH == NULL) {
		FrmAlert(ErrOpenEncAlert);
		return false;
	}
	toCharsetH =
	    DmGetResourceIndex(dbP,
			       DmFindResourceType(dbP, ENC_RES_TYPE,
						  toCharsetIndex));
	if (toCharsetH == NULL) {
		FrmAlert(ErrOpenEncAlert);
		return false;
	}

	MemBuf = MemHandleNew(MEM_BUF_SIZE);
	if (MemBuf == NULL) {
		FrmAlert(NoMemAlert);
		return false;
	}

	ConvTable = (UInt8 *) MemPtrNew(CONV_TABLE_SIZE);
	if (ConvTable == NULL) {
		FrmAlert(NoMemAlert);
		return false;
	}
	p = ConvTable - CONV_TABLE_SIZE;
	MemSet(ConvTable, CONV_TABLE_SIZE, NO_RECODE);

	fromCharsetP = (UInt8 *) MemHandleLock(fromCharsetH);
	toCharsetP = (UInt8 *) MemHandleLock(toCharsetH);

	fromNameSize = StrLen((Char *) fromCharsetP) + 1;
	toNameSize = StrLen((Char *) toCharsetP) + 1;
	pf = fromCharsetP + fromNameSize;
	pt = toCharsetP + toNameSize;
	CharsetSize = MemHandleSize(fromCharsetH) - fromNameSize;
	CharsetSize2 = MemHandleSize(toCharsetH) - toNameSize;
	CharsetSize =
	    (CharsetSize <= CharsetSize2) ? CharsetSize : CharsetSize2;

	for (index = 0; index < CharsetSize; index++, pf++, pt++) {
		p[*pf] = *pt;
	}

	MemPtrUnlock(toCharsetP);
	MemPtrUnlock(fromCharsetP);
	DmReleaseResource(toCharsetH);
	DmReleaseResource(fromCharsetH);
	return true;
}

void ConverterFree()
{
	if (ConvTable)
		MemPtrFree((MemPtr *) ConvTable);
	if (MemBuf)
		MemHandleFree(MemBuf);
}

UInt16 ConvertStr(Char * str, Char * strDest, UInt16 destSize)
{
	UInt8 *strP, *ctp, *strDestP;

	strP = (UInt8 *) str;
	ctp = ConvTable - CONV_TABLE_SIZE;
	strDestP = (UInt8 *) strDest;

	while (*strP && destSize--) {
		if (*strP >= LOWER_BOUND && ctp[*strP] != NO_RECODE) {
			*strDestP = ctp[*strP];
		} else
			*strDestP = *strP;
		strP++;
		strDestP++;
	}
	return strP - (UInt8 *) str;
}

void ConvertDBStr(Char * str, MemPtr recordP)
{
	Char *strDest;
	UInt16 strLen, convLen;

	strLen = StrLen(str);
	if (strLen == 0)
		return;
	strDest = (Char *) MemHandleLock(MemBuf);
	do {
		convLen = ConvertStr(str, strDest, MEM_BUF_SIZE);
		DmWrite(recordP, str - (Char *) recordP, strDest, convLen);
		str += convLen;
		strLen -= convLen;
	}
	while (strLen);
	MemHandleUnlock(MemBuf);
}
