/**************************************************************************

Copyright (c) 2002 Dmitry Vukolov <dvukol@rosmail.ru>

This is free software, licensed under the GNU Public License v2.
See the file COPYING for details.

**************************************************************************/

#include <PalmOS.h>
#include "convert.h"
#include "addressdb.h"
#include "datedb.h"
#include "memodb.h"
#include "tododb.h"
#include "launcherdb.h"
#include "resource.h"


/**********************************************************
        Private definitions:
***********************************************************/
#define APP_CREATOR              'CRcd'

typedef void (*CONVFUNC) (void);

CONVFUNC DBConv[] = {
	AddrConvert,
	DateConvert,
	MemoConvert,
	ToDoConvert,
	LauncherConvert
};

typedef struct {
	UInt8 CheckboxStates;
	UInt32 LastRomVersion;
} TPrefs;
TPrefs prefs;

#define BitAtPosition(pos)              ((UInt32)1 << (pos))
#define GetBitMacro(bitfield, index)    ((bitfield) & BitAtPosition(index))
#define SetBitMacro(bitfield, index)    ((bitfield) |= BitAtPosition(index))

void StartApp();
void StopApp();
void MainFormInit(FormPtr frmP);
Boolean AppHandleEvent(EventPtr event);
Boolean DoMenu(FormPtr frmP, UInt16 command);
Boolean OnOpenEvent(FormPtr frmP);

void *GetObjectPtr(FormPtr frmP, UInt16 objID);
UInt16 CtlGetValueFromID(FormPtr frmP, UInt16 objID);
void CtlSetValueFromID(FormPtr frmP, UInt16 objID, Int16 NewValue);

void EncodingsListInit(FormPtr frmP);
void Convert(FormPtr frmP);

/**********************************************************
        Internal Functions:
***********************************************************/

void MainFormInit(FormPtr frmP)
{
	UInt16 index;

	EncodingsListInit(frmP);
	for (index = 0; index < DB_COUNT; index++) {
		CtlSetValueFromID(frmP, index,
				  GetBitMacro(prefs.CheckboxStates, index));
	}
}

void *GetObjectPtr(FormPtr frmP, UInt16 objID)
{
	return FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objID));
}

UInt16 CtlGetValueFromID(FormPtr frmP, UInt16 objID)
{
	return FrmGetControlValue(frmP, FrmGetObjectIndex(frmP, objID));
}

void CtlSetValueFromID(FormPtr frmP, UInt16 objID, Int16 NewValue)
{
	FrmSetControlValue(frmP, FrmGetObjectIndex(frmP, objID), NewValue);
}

void Convert(FormPtr frmP)
{
	Int16 FromEnc, ToEnc, ctlID;
	ListPtr listP;

	listP = (ListPtr) GetObjectPtr(frmP, FromList);
	FromEnc = LstGetSelection(listP);
	listP = (ListPtr) GetObjectPtr(frmP, ToList);
	ToEnc = LstGetSelection(listP);
	if (FromEnc == noListSelection || ToEnc == noListSelection) {
		FrmAlert(SelectEncsAlert);
		return;
	}
	if (FromEnc == ToEnc) {
		FrmAlert(SameEncsAlert);
		return;
	}
	for (ctlID = 0; ctlID < DB_COUNT; ctlID++)
		if (CtlGetValueFromID(frmP, ctlID))
			break;
	if (ctlID >= DB_COUNT) {
		FrmAlert(SelectDBsAlert);
		return;
	}
	if (FrmAlert(ConfirmAlert) != 0)
		return;

	FrmShowObject(frmP, FrmGetObjectIndex(frmP, StatusLabel));
	if (!ConverterInit(FromEnc, ToEnc)) {
		FrmHideObject(frmP, FrmGetObjectIndex(frmP, StatusLabel));
		return;
	}
	for (ctlID = 0; ctlID < DB_COUNT; ctlID++)
                if(CtlGetValueFromID(frmP, ctlID))
	                (*DBConv[ctlID]) ();
	FrmHideObject(frmP, FrmGetObjectIndex(frmP, StatusLabel));
	FrmAlert(DoneAlert);
}


void EncodingsListInit(FormPtr frmP)
{
	Char **ENames;
	ListPtr listP;
	UInt16 numEncs;

	ENames = GetEncNames();
	if (!ENames) {
		ErrThrow(-1);
		return;
	}

	numEncs = EncTotal();

	listP = (ListPtr) GetObjectPtr(frmP, FromList);
	LstSetListChoices(listP, ENames, numEncs);
	LstSetSelection(listP, -1);

	listP = (ListPtr) GetObjectPtr(frmP, ToList);
	LstSetListChoices(listP, ENames, numEncs);
	LstSetSelection(listP, -1);
}

Boolean DoMenu(FormPtr frmP, UInt16 command)
{
	Boolean handled = false;

	switch (command) {
	case MenuAbout:
		FrmHelp(AboutStr);
		handled = true;
		break;
	}
	return handled;
}

Boolean OnOpenEvent(FormPtr frmP)
{
	UInt32 romVersion;

	MainFormInit(frmP);
	FrmDrawForm(frmP);

	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < 0x02003000) {
		FrmAlert(OSVerNotCompatAlert);
		ErrThrow(-1);
		return false;
	}
	if (romVersion != prefs.LastRomVersion) {
		if (romVersion >= 0x04003000)
			FrmHelp(FormatChangeAlert);
		else
			FrmAlert(SuggestBackupAlert);
		prefs.LastRomVersion = romVersion;
	}
	return true;
}

Boolean mainFormEventHandler(EventPtr eventP)
{
	Boolean handled = false;
	FormPtr frmP = FrmGetActiveForm();
	switch (eventP->eType) {
	case frmOpenEvent:
		handled = OnOpenEvent(frmP);
		break;
	case menuEvent:
		handled = DoMenu(frmP, eventP->data.menu.itemID);
		break;
	case ctlSelectEvent:
		switch (eventP->data.ctlSelect.controlID) {
		case ConvertBtn:
			Convert(frmP);
			handled = true;
			break;
		}
		break;
	default:		// needed to suppress compiler warnings
		break;		// on not handled values in switch
	}
	return handled;
}

void StartApp()
{
	UInt16 prefSize = sizeof(TPrefs);
	if ((PrefGetAppPreferences(APP_CREATOR, 0, &prefs, &prefSize, true)
	     == noPreferenceFound) || (prefSize != sizeof(TPrefs))) {
		// default initialization, since discovered Prefs was missing or old.
		prefs.CheckboxStates = 255;
		prefs.LastRomVersion = 0;
	}
}

void StopApp()
{
	UInt16 index;
	FormPtr frmP;

	frmP = FrmGetActiveForm();
	prefs.CheckboxStates = 0;

	for (index = 0; index < DB_COUNT; index++) {
		if (CtlGetValueFromID(frmP, index))
			SetBitMacro(prefs.CheckboxStates, index);
	}

	PrefSetAppPreferences(APP_CREATOR, 0, 1, &prefs, sizeof(TPrefs), true);

	ConverterFree();
	FreeEncNames();
}

Boolean AppHandleEvent(EventPtr event)
{
	FormPtr frm;
	UInt16 formId;
	Boolean handled = false;
	if (event->eType == frmLoadEvent) {
		formId = event->data.frmLoad.formID;
		frm = FrmInitForm(formId);
		FrmSetActiveForm(frm);
		if (formId == MainForm)
			FrmSetEventHandler(frm, mainFormEventHandler);
		handled = true;
	}
	return handled;
}

UInt32 PilotMain(UInt16 cmd, void *cmdPBP, UInt16 launchFlags)
{
	EventType event;
	UInt16 error;
	if (cmd == sysAppLaunchCmdNormalLaunch) {
		ErrTry {
			StartApp();
			FrmGotoForm(MainForm);
			do {
				EvtGetEvent(&event, evtWaitForever);
				if (!(SysHandleEvent(&event)
				    || MenuHandleEvent(0, &event, &error)
				    || AppHandleEvent(&event)))
					FrmDispatchEvent(&event);
			}
			while (event.eType != appStopEvent);
		}
		ErrCatch(err) {
			err = err;	// suppress compiler warning
		} ErrEndCatch;
		StopApp();
		FrmCloseAllForms();
	}
	return 0;
}
